package kcas
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
Module Kcas.W1Source
The type of shared memory reference.
cas r e u updates the reference r to value u if the current content of r is e.
try_map r f invokes f c, where c is the result of get r. If the result of f c is None, then Aborted is returned. If the result of f c is Some v, then attempt to CAS update r from c to v. If the CAS succeeds, then Success c is returned. If the CAS fails, then Failed is returned.
Like try_map but retries on CAS failure. Hence, map r f never returns Failed.